#!/usr/bin/ruby
#

require 'set'

##
# Gives the set of all public instance methods of the given class.
def methods_of cls
  cls.public_instance_methods.to_set
end

##
# This is the main function of the CLI.
# Its parameter is an arbitrary class, whose public instance methods
# will be the CLI commands.
#
def main executorCls
  executor = executorCls.new
  cmds = (methods_of(executorCls) - methods_of(Object)).sort
  cmds.map! {|x| x.to_s}

  issued = ARGV.shift
  if not cmds.include? issued
    puts "No command '#{issued}' is known."
    die "Please select from: #{cmds.join ', '}"
  end
  
  task = executorCls.public_instance_method(issued) .bind(executor)
  begin
    task.call *ARGV
  rescue ArgumentError => e
    puts "Try a different parameter set.." # show cmd help
    puts e
  rescue Interrupt
    puts
  end
end

##
# Confirmation on STDIN
def confirm
  printf "Are you sure? [Y/n] "
  answer = STDIN.gets.strip
  if ("Y" != answer)
    die "Command ignored."
  end
end

##
# Displays the given +text+, if any, then quits.
def die text=""
  puts text
  exit
end
